@@ -1,17 +1,18 @@
 # ett_parser.py
 
-from typing import List
+from typing import List, Optional, Iterator
 
 import requests
 import sys
 import aiohttp
 import asyncio
+import pandas as pd
 
 
 def __url(path, legacy_api=False) -> str:
     return (
-               "https://www.elevenvr.club/" if legacy_api else "https://elevenvr.club/api/v1/"
-           ) + path
+        "https://www.elevenvr.club/" if legacy_api else "https://elevenvr.club/api/v1/"
+    ) + path
 
 
 def __exception_handler(func):
@@ -38,7 +39,12 @@ def get_user(user_id) -> List:
     def request_user(user_id):
         return requests.get(__url(f"accounts/{user_id}/", legacy_api=True))
 
-    return request_user(user_id).json()["data"]["attributes"]
+    res = request_user(user_id)
+    if res is None:
+        print(f"Player with id{user_id} not found.")
+        return [None]
+    else:
+        return res.json()["data"]["attributes"]
 
 
 def get_friends(user_id) -> List:
@@ -48,6 +54,7 @@ def get_friends(user_id) -> List:
 
     return request_friends(user_id).json()["data"]
 
+
 def get_matches(user_id, unranked=False) -> List:
     async def request_matches(session, url):
         async with session.get(url) as resp:
@@ -56,17 +63,21 @@ def get_matches(user_id, unranked=False) -> List:
 
     async def get_matches_async(user_id, unranked):
         unranked_str = "true" if unranked else "false"
-        async with aiohttp.ClientSession() as session:
+        async with aiohttp.ClientSession(
+            headers={"Connection": "keep-alive"}
+        ) as session:
             tasks = []
-            res = requests.get(__url(f"accounts/{user_id}/matches/?unranked={unranked_str}")).json()
+            res = requests.get(
+                __url(f"accounts/{user_id}/matches/?unranked={unranked_str}")
+            ).json()
             last_page = res["links"]["last"]
-            num_pages = int(last_page.split('page%5Bnumber%5D=')[1].split('&')[0])
+            num_pages = int(last_page.split("page%5Bnumber%5D=")[1].split("&")[0])
             if num_pages == 1:
                 matches = [res["data"]]
             else:
                 for number in range(1, num_pages + 1):
-                    url = (
-                        __url(f"accounts/{user_id}/matches/?page%5Bnumber%5D={number}&unranked={unranked_str}")
+                    url = __url(
+                        f"accounts/{user_id}/matches/?page%5Bnumber%5D={number}&unranked={unranked_str}"
                     )
                     tasks.append(asyncio.ensure_future(request_matches(session, url)))
 
@@ -91,6 +102,59 @@ def get_elo_history(user_id) -> List:
     return request_elo_history(user_id).json()["data"]
 
 
+def get_matchup(user_id1, user_id2) -> Iterator:
+    # TODO: webapi currently has a bug as it always returns 0 for home-score and away-score
+    @__exception_handler
+    def request_matchup(user_id1, user_id2):
+        return requests.get(__url(f"matchup/{user_id1}/{user_id2}", legacy_api=True))
+
+    r = request_matchup(user_id1, user_id2).json()
+    res = r["data"]
+    if len(res) == 0:
+        return []
+
+    rounds_data = r["included"]
+    rounds_indexed = {}
+
+    for r in rounds_data:
+        rounds_indexed[r["id"]] = r["attributes"]
+
+    for match_data in res:
+        match = match_data["attributes"]
+        match["winning-team"] = match.pop("winner")
+        match["losing-team"] = 1 - match["winning-team"]
+
+        # rounds data
+        rounds = match_data["relationships"]["rounds"]["data"]
+        num_rounds = len(rounds)
+        match["rounds"] = []
+
+        for i in range(num_rounds):
+            current_round = rounds_indexed[
+                match_data["relationships"]["rounds"]["data"][i]["id"]
+            ]
+            match["rounds"].append(current_round)
+            match["rounds"][i]["winner"] = (
+                0 if current_round["home-score"] > current_round["away-score"] else 1
+            )
+
+        # players data
+        match["players"] = []
+        home_away = ["home-team", "away-team"]
+        for i in range(2):
+            match["players"].append(match[home_away[i]][0])
+            match["players"][i]["username"] = match[home_away[i]][0]["UserName"]
+            match["players"][i]["elo"] = match[home_away[i]][0]["ELO"]
+            match["players"][i]["rank"] = match[home_away[i]][0]["Rank"]
+            match["players"][i]["wins"] = match[home_away[i]][0]["Wins"]
+            match["players"][i]["losses"] = match[home_away[i]][0]["Losses"]
+            match["players"][i]["last-online"] = match[home_away[i]][0]["LastOnline"]
+            match["players"][i]["team"] = i
+
+        match_data["attributes"] = match
+        yield match_data
+
+
 def get_leaderboard(num_players=10):
     async def request_leaderboard(session, url):
         async with session.get(url) as resp:
@@ -102,9 +166,9 @@ def get_leaderboard(num_players=10):
             tasks = []
             for number in range(0, num_players // 10 + 1):
                 url = (
-                        __url(f"leaderboards/", legacy_api=True)
-                        + "?start="
-                        + str(number * 10)
+                    __url(f"leaderboards/", legacy_api=True)
+                    + "?start="
+                    + str(number * 10)
                 )
                 tasks.append(asyncio.ensure_future(request_leaderboard(session, url)))
 
@@ -119,6 +183,19 @@ def get_leaderboard(num_players=10):
     return all_pages[:num_players]
 
 
+def get_leaderboard_official_tournament() -> List:
+    @__exception_handler
+    def request_leaderboard_official_tournament():
+        return requests.get("http://lavadesignstudio.co.uk/eleven-rankings/")
+
+    res = request_leaderboard_official_tournament()
+    if res is None:
+        print(f"Official Tournament Leaderboard Not Found.")
+        return [None]
+    else:
+        return pd.read_html(res.text)
+
+
 def user_search(username) -> List:
     async def request_user(session, url):
         async with session.get(url) as resp:
@@ -128,15 +205,18 @@ def user_search(username) -> List:
     async def get_user_async(username):
         async with aiohttp.ClientSession() as session:
             tasks = []
-            res = requests.get(__url(f"accounts/search/{username}/", legacy_api=True)).json()
+            res = requests.get(
+                __url(f"accounts/search/{username}/", legacy_api=True)
+            ).json()
             last_page = res["links"]["last"]
-            num_pages = int(last_page.split('page%5Bnumber%5D=')[1].split('&')[0])
+            num_pages = int(last_page.split("page%5Bnumber%5D=")[1].split("&")[0])
             if num_pages == 1:
                 users = [res["data"]]
             else:
                 for number in range(1, num_pages + 1):
-                    url = (
-                        __url(f"accounts/search/{username}?page%5Bnumber%5D={number}", legacy_api=True)
+                    url = __url(
+                        f"accounts/search/{username}?page%5Bnumber%5D={number}",
+                        legacy_api=True,
                     )
                     tasks.append(asyncio.ensure_future(request_user(session, url)))
 
